import {
  Button,
  Paper,
  TextField,
  Select,
  MenuItem,
  FormControl,
  InputLabel,
  Box,
} from "@mui/material";
import { observer } from "mobx-react-lite";
import { ArrowLeft, Save, ImagePlus } from "lucide-react";
import { Loader2 } from "lucide-react";
import { useNavigate } from "react-router-dom";
import { toast } from "react-toastify";
import { cityStore, countryStore, mediaStore } from "@shared";
import { useState, useEffect } from "react";
import { LanguageSwitcher, MediaViewer } from "@widgets";
import { SelectMediaDialog } from "@shared";

export const CityCreatePage = observer(() => {
  const navigate = useNavigate();
  const [name, setName] = useState("");
  const [countryCode, setCountryCode] = useState("");
  const [selectedMediaId, setSelectedMediaId] = useState<string | null>(null);
  const [isLoading, setIsLoading] = useState(false);
  const [isSelectMediaOpen, setIsSelectMediaOpen] = useState(false);

  useEffect(() => {
    countryStore.getCountries();
    mediaStore.getMedia();
  }, []);

  const handleCreate = async () => {
    try {
      setIsLoading(true);
      await cityStore.createCity(
        name,
        countryStore.countries.find((c) => c.code === countryCode)?.name!,
        countryCode,
        selectedMediaId!
      );
      toast.success("Город успешно создан");
      navigate("/city");
    } catch (error) {
      toast.error("Ошибка при создании города");
    } finally {
      setIsLoading(false);
    }
  };

  const handleMediaSelect = (media: {
    id: string;
    filename: string;
    media_name?: string;
    media_type: number;
  }) => {
    setSelectedMediaId(media.id);
  };

  const selectedMedia = selectedMediaId
    ? mediaStore.media.find((m) => m.id === selectedMediaId)
    : null;

  return (
    <Paper className="w-full h-full p-3 flex flex-col gap-10">
      <LanguageSwitcher />
      <div className="flex items-center gap-4">
        <button
          className="flex items-center gap-2"
          onClick={() => navigate("/city")}
        >
          <ArrowLeft size={20} />
          Назад
        </button>
      </div>

      <div className="flex flex-col gap-10 w-full items-end">
        <TextField
          fullWidth
          label="Название города"
          value={name}
          required
          onChange={(e) => setName(e.target.value)}
        />

        <FormControl fullWidth>
          <InputLabel>Страна</InputLabel>
          <Select
            value={countryCode}
            label="Страна"
            required
            onChange={(e) => setCountryCode(e.target.value)}
          >
            {countryStore.countries.map((country) => (
              <MenuItem key={country.code} value={country.code}>
                {country.name}
              </MenuItem>
            ))}
          </Select>
        </FormControl>

        <div className="w-full flex flex-col gap-4">
          <label className="text-sm text-gray-600">Герб города</label>
          <div className="flex items-center gap-4">
            <Button
              variant="outlined"
              onClick={() => setIsSelectMediaOpen(true)}
              startIcon={<ImagePlus size={20} />}
            >
              Выбрать герб
            </Button>
            {selectedMedia && (
              <span className="text-sm text-gray-600">
                {selectedMedia.media_name || selectedMedia.filename}
              </span>
            )}
          </div>
          {selectedMedia && (
            <Box
              sx={{
                width: "200px",
                height: "200px",
                border: "1px solid #e0e0e0",
                borderRadius: "8px",
                overflow: "hidden",
                display: "flex",
                alignItems: "center",
                justifyContent: "center",
              }}
            >
              <MediaViewer
                media={{
                  id: selectedMedia.id,
                  media_type: selectedMedia.media_type,
                  filename: selectedMedia.filename,
                }}
              />
            </Box>
          )}
        </div>

        <Button
          variant="contained"
          className="w-min flex gap-2 items-center"
          startIcon={<Save size={20} />}
          onClick={handleCreate}
          disabled={isLoading || !name || !countryCode}
        >
          {isLoading ? (
            <Loader2 size={20} className="animate-spin" />
          ) : (
            "Создать"
          )}
        </Button>
      </div>

      <SelectMediaDialog
        open={isSelectMediaOpen}
        onClose={() => setIsSelectMediaOpen(false)}
        onSelectMedia={handleMediaSelect}
        mediaType={3} // Тип медиа для иконок
      />
    </Paper>
  );
});
