import React, {
  useEffect,
  useRef,
  useState,
  useCallback,
  ReactNode,
} from "react";
import { Map, View, Overlay, MapBrowserEvent } from "ol";
import TileLayer from "ol/layer/Tile";
import OSM from "ol/source/OSM";
import VectorLayer from "ol/layer/Vector";
import VectorSource, { VectorSourceEvent } from "ol/source/Vector";
import { Draw, Modify, Select } from "ol/interaction";
import { DrawEvent } from "ol/interaction/Draw";
import { ModifyEvent } from "ol/interaction/Modify";
import { SelectEvent } from "ol/interaction/Select";
import { Style, Fill, Stroke, Circle as CircleStyle } from "ol/style";
import { Point, LineString, Geometry } from "ol/geom";
import { transform } from "ol/proj";
import { GeoJSON } from "ol/format";
import { Bus, RouteIcon, MapPin, Trash2, ArrowRightLeft } from "lucide-react";
import { altKeyOnly, primaryAction, singleClick } from "ol/events/condition";
import { Feature } from "ol";
import Layer from "ol/layer/Layer";
import Source from "ol/source/Source";
import LayerRenderer from "ol/renderer/Layer";
import { FeatureLike } from "ol/Feature";

// --- CONFIGURATION ---
export const mapConfig = {
  center: [30.311, 59.94] as [number, number],
  zoom: 13,
};

// --- SVG ICONS ---
const EditIcon = () => (
  <svg
    xmlns="http://www.w3.org/2000/svg"
    className="h-4 w-4 mr-1 sm:mr-2"
    fill="none"
    viewBox="0 0 24 24"
    stroke="currentColor"
  >
    <path
      strokeLinecap="round"
      strokeLinejoin="round"
      strokeWidth={2}
      d="M15.232 5.232l3.536 3.536m-2.036-5.036a2.5 2.5 0 113.536 3.536L6.5 21.036H3v-3.572L16.732 3.732z"
    />
  </svg>
);
const StatsIcon = () => (
  <svg
    xmlns="http://www.w3.org/2000/svg"
    className="h-4 w-4 mr-1 sm:mr-2"
    fill="none"
    viewBox="0 0 24 24"
    stroke="currentColor"
  >
    <path
      strokeLinecap="round"
      strokeLinejoin="round"
      strokeWidth={2}
      d="M16 8v8m-4-5v5m-4-2v2m-2 4h12a2 2 0 002-2V6a2 2 0 00-2-2H6a2 2 0 00-2 2v12a2 2 0 002 2z"
    />
  </svg>
);
const LineIconSvg = () => (
  <svg
    xmlns="http://www.w3.org/2000/svg"
    className="h-4 w-4 mr-1 sm:mr-2"
    fill="none"
    viewBox="0 0 24 24"
    stroke="currentColor"
  >
    <path
      strokeLinecap="round"
      strokeLinejoin="round"
      strokeWidth={2}
      d="M3.055 11H5a2 2 0 012 2v1a2 2 0 002 2h10a2 2 0 002-2v-1a2 2 0 012-2h1.945M7.732 4.064A2.5 2.5 0 105.23 6.24m13.54 0a2.5 2.5 0 10-2.502-2.176M12 16.05V21m0-17.948V3"
    />
  </svg>
);

// --- TYPE DEFINITIONS ---
interface MapServiceConfig {
  target: HTMLElement;
  center: [number, number];
  zoom: number;
}

interface HistoryState {
  action: string;
  state: string; // GeoJSON string
}

// Internal OL type extensions for better type safety with private/protected members

class MapService {
  private map: Map | null;
  private vectorSource: VectorSource<Feature<Geometry>>;
  private vectorLayer: VectorLayer<VectorSource<Feature<Geometry>>>;
  private tooltipElement: HTMLElement;
  private tooltipOverlay: Overlay | null;
  private mode: string | null;
  private currentDrawingType: string | null;
  private currentInteraction: Draw | null;
  private modifyInteraction: Modify;
  private selectInteraction: Select;
  private infoSelectedFeatureId: string | number | null;
  private hoveredFeatureId: string | number | null;
  private history: HistoryState[];
  private historyIndex: number;
  private beforeModifyState: string | null; // GeoJSON string
  private boundHandlePointerMove: (
    event: MapBrowserEvent<PointerEvent>
  ) => void;
  private boundHandlePointerLeave: () => void;
  private boundHandleContextMenu: (event: MouseEvent) => void;
  private boundHandleKeyDown: (event: KeyboardEvent) => void;

  // Styles
  private defaultStyle: Style;
  private selectedStyle: Style;
  private drawStyle: Style;
  private infoSelectedLineStyle: Style;
  private busIconStyle: Style;
  private selectedBusIconStyle: Style;
  private drawBusIconStyle: Style;
  private infoSelectedBusIconStyle: Style;
  private universalHoverStylePoint: Style;
  private universalHoverStyleLine: Style;

  // Callbacks
  private setCoordinatesPanelContent: (content: string) => void;
  private setShowCoordinatesPanel: (show: boolean) => void;
  private setLoading: (loading: boolean) => void;
  private setError: (error: string | null) => void;
  private onModeChangeCallback: (mode: string) => void;
  private onFeaturesChange: (features: Feature<Geometry>[]) => void;
  private onFeatureSelect: (feature: Feature<Geometry> | null) => void;

  constructor(
    config: MapServiceConfig,
    setCoordinatesPanelContent: (content: string) => void,
    setShowCoordinatesPanel: (show: boolean) => void,
    setLoading: (loading: boolean) => void,
    setError: (error: string | null) => void,
    onModeChangeCallback: (mode: string) => void,
    onFeaturesChange: (features: Feature<Geometry>[]) => void,
    onFeatureSelect: (feature: Feature<Geometry> | null) => void,
    tooltipElement: HTMLElement
  ) {
    this.map = null;
    this.tooltipElement = tooltipElement;
    this.tooltipOverlay = null;
    this.mode = null;
    this.currentDrawingType = null;
    this.currentInteraction = null;
    this.infoSelectedFeatureId = null;
    this.hoveredFeatureId = null;
    this.history = [];
    this.historyIndex = -1;
    this.beforeModifyState = null;

    this.setCoordinatesPanelContent = setCoordinatesPanelContent;
    this.setShowCoordinatesPanel = setShowCoordinatesPanel;
    this.setLoading = setLoading;
    this.setError = setError;
    this.onModeChangeCallback = onModeChangeCallback;
    this.onFeaturesChange = onFeaturesChange;
    this.onFeatureSelect = onFeatureSelect;

    this.defaultStyle = new Style({
      fill: new Fill({ color: "rgba(66, 153, 225, 0.2)" }),
      stroke: new Stroke({ color: "#3182ce", width: 2 }),
    });
    this.selectedStyle = new Style({
      fill: new Fill({ color: "rgba(221, 107, 32, 0.3)" }),
      stroke: new Stroke({ color: "#dd6b20", width: 3 }),
      image: new CircleStyle({
        radius: 6,
        fill: new Fill({ color: "#dd6b20" }),
        stroke: new Stroke({ color: "white", width: 1.5 }),
      }),
    });
    this.drawStyle = new Style({
      fill: new Fill({ color: "rgba(74, 222, 128, 0.3)" }),
      stroke: new Stroke({
        color: "rgba(34, 197, 94, 0.7)",
        width: 2,
        lineDash: [5, 5],
      }),
      image: new CircleStyle({
        radius: 5,
        fill: new Fill({ color: "rgba(34, 197, 94, 0.7)" }),
      }),
    });
    this.infoSelectedLineStyle = new Style({
      fill: new Fill({ color: "rgba(255, 0, 0, 0.2)" }),
      stroke: new Stroke({ color: "rgba(255, 0, 0, 0.9)", width: 3 }),
    });
    this.busIconStyle = new Style({
      image: new CircleStyle({
        radius: 8,
        fill: new Fill({ color: "rgba(0, 60, 255, 0.8)" }),
        stroke: new Stroke({ color: "#ffffff", width: 1.5 }),
      }),
    });
    this.selectedBusIconStyle = new Style({
      image: new CircleStyle({
        radius: 10,
        fill: new Fill({ color: "rgba(221, 107, 32, 0.9)" }),
        stroke: new Stroke({ color: "#ffffff", width: 2 }),
      }),
    });
    this.drawBusIconStyle = new Style({
      image: new CircleStyle({
        radius: 8,
        fill: new Fill({ color: "rgba(100, 180, 100, 0.8)" }),
        stroke: new Stroke({ color: "#ffffff", width: 1.5 }),
      }),
    });
    this.infoSelectedBusIconStyle = new Style({
      image: new CircleStyle({
        radius: 10,
        fill: new Fill({ color: "rgba(255, 0, 0, 0.9)" }),
        stroke: new Stroke({ color: "#ffffff", width: 2 }),
      }),
    });
    this.universalHoverStylePoint = new Style({
      image: new CircleStyle({
        radius: 11,
        fill: new Fill({ color: "rgba(255, 165, 0, 0.7)" }),
        stroke: new Stroke({ color: "rgba(255,255,255,0.8)", width: 2 }),
      }),
    });
    this.universalHoverStyleLine = new Style({
      fill: new Fill({ color: "rgba(255, 165, 0, 0.3)" }),
      stroke: new Stroke({ color: "rgba(255, 165, 0, 0.8)", width: 3.5 }),
    });

    this.vectorSource = new VectorSource<Feature<Geometry>>();
    this.vectorLayer = new VectorLayer({
      source: this.vectorSource,
      style: (featureLike: FeatureLike) => {
        const feature = featureLike as Feature<Geometry>; // We know our source has Features
        const geometryType = feature.getGeometry()?.getType();
        const fId = feature.getId();
        const isEditSelected = this.selectInteraction
          ?.getFeatures()
          .getArray()
          .includes(feature);
        const isInfoSelected =
          this.infoSelectedFeatureId === fId && this.mode === "statistics";
        const isHovered = this.hoveredFeatureId === fId;

        if (geometryType === "Point") {
          if (isHovered && !isEditSelected && !isInfoSelected)
            return this.universalHoverStylePoint;
          if (isInfoSelected) return this.infoSelectedBusIconStyle;
          return isEditSelected ? this.selectedBusIconStyle : this.busIconStyle;
        } else if (geometryType === "LineString") {
          if (isHovered && !isEditSelected && !isInfoSelected)
            return this.universalHoverStyleLine;
          if (isInfoSelected) return this.infoSelectedLineStyle;
          return isEditSelected ? this.selectedStyle : this.defaultStyle;
        }
        return this.defaultStyle;
      },
    });

    this.boundHandlePointerMove = this.handlePointerMove.bind(this);
    this.boundHandlePointerLeave = this.handlePointerLeave.bind(this);
    this.boundHandleContextMenu = this.handleContextMenu.bind(this);
    this.boundHandleKeyDown = this.handleKeyDown.bind(this);

    this.vectorSource.on(
      "addfeature",
      this.handleFeatureEvent.bind(this) as (
        event: VectorSourceEvent<Feature<Geometry>>
      ) => void
    );
    this.vectorSource.on("removefeature", () => this.updateFeaturesInReact());
    this.vectorSource.on(
      "changefeature",
      this.handleFeatureChange.bind(this) as (
        event: VectorSourceEvent<Feature<Geometry>>
      ) => void
    );

    let renderCompleteHandled = false;
    const MAP_LOAD_TIMEOUT = 15000;
    try {
      this.map = new Map({
        target: config.target,
        layers: [new TileLayer({ source: new OSM() }), this.vectorLayer],
        view: new View({
          center: transform(config.center, "EPSG:4326", "EPSG:3857"),
          zoom: config.zoom,
        }),
        controls: [],
      });
      if (this.tooltipElement && this.map) {
        this.tooltipOverlay = new Overlay({
          element: this.tooltipElement,
          offset: [15, 0],
          positioning: "center-left",
        });
        this.map.addOverlay(this.tooltipOverlay);
      }
      this.map.once("rendercomplete", () => {
        if (!renderCompleteHandled) {
          this.setLoading(false);
          renderCompleteHandled = true;
          this.setError(null);
        }
      });
      setTimeout(() => {
        if (!renderCompleteHandled && this.map) {
          this.setLoading(false);
          this.setError("Карта не загрузилась вовремя.");
          renderCompleteHandled = true;
        }
      }, MAP_LOAD_TIMEOUT);
    } catch (error) {
      this.setError("Критическая ошибка при инициализации карты.");
      this.setLoading(false);
      renderCompleteHandled = true;
    }

    this.modifyInteraction = new Modify({
      source: this.vectorSource,
      style: (f: FeatureLike) =>
        f.getGeometry()?.getType() === "Point"
          ? this.selectedBusIconStyle
          : this.selectedStyle,
      deleteCondition: (
        e: MapBrowserEvent<PointerEvent | KeyboardEvent | WheelEvent>
      ) =>
        altKeyOnly(e as MapBrowserEvent<PointerEvent>) &&
        primaryAction(e as MapBrowserEvent<PointerEvent>),
    });

    this.selectInteraction = new Select({
      style: (featureLike: FeatureLike) => {
        if (!featureLike || !featureLike.getGeometry) return this.defaultStyle;
        const feature = featureLike as Feature<Geometry>;
        const geometryType = feature.getGeometry()?.getType();
        return geometryType === "Point"
          ? this.selectedBusIconStyle
          : this.selectedStyle;
      },
      condition: (
        e: MapBrowserEvent<PointerEvent | KeyboardEvent | WheelEvent>
      ) => {
        const isEdit = this.mode === "edit";
        const isSingle = singleClick(e as MapBrowserEvent<PointerEvent>);
        if (!isEdit || !isSingle) return false;

        let clickModify = false;
        if (this.modifyInteraction.getActive() && this.map) {
          const px = e.pixel;
          const internalModify = this.modifyInteraction as Modify;
          const sketchFs: Feature<Point>[] | undefined = (
            internalModify as unknown as {
              overlay_?: VectorLayer<VectorSource<Feature<Point>>>;
            }
          ).overlay_
            ?.getSource()
            ?.getFeatures();

          if (sketchFs) {
            for (const sf of sketchFs) {
              const g = sf.getGeometry();
              if (g) {
                const coord = this.map.getCoordinateFromPixel(px);
                if (!coord) continue;
                const cp = g.getClosestPoint(coord);
                const cppx = this.map.getPixelFromCoordinate(cp);
                if (!cppx) continue;
                const pixelTolerance =
                  (internalModify as unknown as { pixelTolerance_?: number })
                    .pixelTolerance_ || 10;
                if (
                  Math.sqrt((px[0] - cppx[0]) ** 2 + (px[1] - cppx[1]) ** 2) <
                  pixelTolerance + 2
                ) {
                  clickModify = true;
                  break;
                }
              }
            }
          }
        }
        return !clickModify;
      },
      filter: (
        _: FeatureLike,
        l: Layer<
          Source,
          LayerRenderer<VectorLayer<VectorSource<Feature<Geometry>>>>
        > | null
      ) => l === this.vectorLayer,
    });

    this.modifyInteraction.on("modifystart", () => {
      const geoJSONFormat = new GeoJSON();
      if (!this.map) return;
      this.beforeModifyState = geoJSONFormat.writeFeatures(
        this.vectorSource.getFeatures(),
        {
          dataProjection: "EPSG:4326",
          featureProjection: this.map.getView().getProjection().getCode(),
        }
      );
    });

    this.modifyInteraction.on("modifyend", (event: ModifyEvent) => {
      if (this.beforeModifyState) {
        this.addStateToHistory("modify-before", this.beforeModifyState);
        this.beforeModifyState = null;
      }
      this.updateFeaturesInReact();
      event.features.forEach((f) => {
        if (f instanceof Feature) {
          // Ensure f is a Feature instance
          this.getFeatureCoordinates(f as Feature<Geometry>);
        }
      });
    });

    if (this.map) {
      this.map.addInteraction(this.modifyInteraction);
      this.map.addInteraction(this.selectInteraction);
      this.modifyInteraction.setActive(false);
      this.selectInteraction.setActive(false);

      this.selectInteraction.on("select", (e: SelectEvent) => {
        if (this.mode === "edit") {
          this.infoSelectedFeatureId = null;
          this.vectorLayer.changed();
          const selFs = e.selected as Feature<Geometry>[]; // Selected features are always ol/Feature
          this.modifyInteraction.setActive(selFs.length > 0);
          if (selFs.length > 0) {
            this.getFeatureCoordinates(selFs[0]);
            this.onFeatureSelect(selFs[0]);
          } else {
            this.hideCoordinatesPanel();
            this.onFeatureSelect(null);
          }
        } else if (this.mode === "statistics") {
          this.onFeatureSelect(null); // Clear selection when switching to stats mode via click
        }
      });
      this.map.on("click", this.handleClick.bind(this) as any);
      this.map.on("pointermove", this.boundHandlePointerMove as any);
      const targetEl = this.map.getTargetElement();
      if (targetEl instanceof HTMLElement) {
        targetEl.addEventListener("contextmenu", this.boundHandleContextMenu);
        targetEl.addEventListener("pointerleave", this.boundHandlePointerLeave);
      }
      document.addEventListener("keydown", this.boundHandleKeyDown);
      this.activateEditMode();
    }
  }

  private addStateToHistory(
    actionDescription: string,
    stateToSave: string
  ): void {
    this.history = this.history.slice(0, this.historyIndex + 1);
    this.history.push({ action: actionDescription, state: stateToSave });
    this.historyIndex = this.history.length - 1;
  }

  private getCurrentStateAsGeoJSON(): string | null {
    if (!this.map) return null;
    const geoJSONFormat = new GeoJSON();
    return geoJSONFormat.writeFeatures(this.vectorSource.getFeatures(), {
      dataProjection: "EPSG:4326",
      featureProjection: this.map.getView().getProjection().getCode(),
    });
  }

  public undo(): void {
    if (this.historyIndex >= 0) {
      const stateToRestore = this.history[this.historyIndex].state;
      this.applyHistoryState(stateToRestore);
      this.historyIndex--;
    } else {
      this.vectorSource.clear();
      this.updateFeaturesInReact();
      this.onFeatureSelect(null);
      this.hideCoordinatesPanel();
    }
  }

  public redo(): void {
    if (this.historyIndex < this.history.length - 1) {
      this.historyIndex++;
      this.applyHistoryState(this.history[this.historyIndex].state);
    }
  }

  private applyHistoryState(geoJSONState: string): void {
    if (!this.map) return;
    const geoJSONFormat = new GeoJSON();
    const features = geoJSONFormat.readFeatures(geoJSONState, {
      dataProjection: "EPSG:4326",
      featureProjection: this.map.getView().getProjection().getCode(),
    }) as Feature<Geometry>[];
    this.vectorSource.clear();
    if (features.length > 0) this.vectorSource.addFeatures(features);
    this.updateFeaturesInReact();
    this.onFeatureSelect(null);
    this.hideCoordinatesPanel();
    this.selectInteraction.getFeatures().clear();
    this.infoSelectedFeatureId = null;
    this.vectorLayer.changed();
  }

  private updateFeaturesInReact(): void {
    if (this.onFeaturesChange) {
      this.onFeaturesChange(this.vectorSource.getFeatures());
    }
  }

  private handlePointerLeave(): void {
    if (this.hoveredFeatureId) {
      this.hoveredFeatureId = null;
      this.vectorLayer.changed();
    }
    if (this.tooltipOverlay) this.tooltipOverlay.setPosition(undefined);
  }

  private handleKeyDown(event: KeyboardEvent): void {
    if ((event.ctrlKey || event.metaKey) && event.key === "z") {
      event.preventDefault();
      this.undo();
      return;
    }
    if ((event.ctrlKey || event.metaKey) && event.key === "y") {
      event.preventDefault();
      this.redo();
      return;
    }
    if (event.key === "Escape") {
      if (this.mode && this.mode.startsWith("drawing-")) this.finishDrawing();
      else if (
        this.mode === "edit" &&
        this.selectInteraction?.getFeatures().getLength() > 0
      ) {
        this.selectInteraction.getFeatures().clear();
        // Firing an event for the selection change might be good here
        // So that the UI (like coordinates panel) updates
        this.onFeatureSelect(null);
        this.hideCoordinatesPanel();
      } else if (this.mode === "statistics" && this.infoSelectedFeatureId) {
        this.infoSelectedFeatureId = null;
        this.vectorLayer.changed();
        this.hideCoordinatesPanel();
        this.onFeatureSelect(null);
      }
    }
  }

  private setMode(newMode: string): void {
    if (!this.map) return;
    const oldMode = this.mode;
    this.mode = newMode;

    if (this.hoveredFeatureId && oldMode !== newMode) {
      this.hoveredFeatureId = null;
      this.vectorLayer.changed();
      if (this.tooltipOverlay) this.tooltipOverlay.setPosition(undefined);
    }

    if (newMode !== "statistics" && this.infoSelectedFeatureId) {
      this.infoSelectedFeatureId = null;
      this.vectorLayer.changed();
    }

    if (this.onModeChangeCallback) this.onModeChangeCallback(newMode);

    if (this.currentInteraction instanceof Draw) {
      this.map.removeInteraction(this.currentInteraction);
      this.currentInteraction = null;
    }

    if (newMode === "edit") {
      this.selectInteraction.setActive(true);
    } else {
      this.selectInteraction.getFeatures().clear();
      this.selectInteraction.setActive(false);
      this.modifyInteraction.setActive(false);
    }

    const isEditWithSelection =
      newMode === "edit" &&
      this.selectInteraction?.getFeatures().getLength() > 0;
    const isStatsWithSelection =
      newMode === "statistics" && this.infoSelectedFeatureId;

    if (!isEditWithSelection && !isStatsWithSelection) {
      this.hideCoordinatesPanel();
    }
  }

  public activateEditMode(): void {
    this.currentDrawingType = null;
    this.setMode("edit");
    if (this.selectInteraction.getFeatures().getLength() > 0) {
      const firstSelectedFeature = this.selectInteraction
        .getFeatures()
        .item(0) as Feature<Geometry>;
      this.getFeatureCoordinates(firstSelectedFeature);
    } else {
      this.hideCoordinatesPanel();
    }
  }

  public activateStatisticsMode(): void {
    this.currentDrawingType = null;
    this.setMode("statistics");
    if (this.selectInteraction?.getFeatures().getLength() > 0) {
      this.selectInteraction.getFeatures().clear();
    }
    if (this.infoSelectedFeatureId) {
      const feature = this.vectorSource.getFeatureById(
        this.infoSelectedFeatureId
      );
      if (feature) this.getFeatureCoordinates(feature);
    } else {
      this.hideCoordinatesPanel();
    }
  }

  public startDrawing(type: "Point" | "LineString", options = {}): void {
    if (!this.map) return;
    this.currentDrawingType = type;
    const drawingMode = `drawing-${type.toLowerCase()}`;
    this.setMode(drawingMode);
    if (this.currentInteraction instanceof Draw) {
      this.map.removeInteraction(this.currentInteraction);
    }

    this.currentInteraction = new Draw({
      source: this.vectorSource,
      type,
      style: type === "Point" ? this.drawBusIconStyle : this.drawStyle,
      ...options,
    });

    let stateBeforeDraw: string | null = null;
    this.currentInteraction.on("drawstart", () => {
      stateBeforeDraw = this.getCurrentStateAsGeoJSON();
    });

    this.currentInteraction.on("drawend", (event: DrawEvent) => {
      if (stateBeforeDraw) {
        this.addStateToHistory("draw-before", stateBeforeDraw);
      }
      const feature = event.feature as Feature<Geometry>; // DrawEvent feature is always an ol/Feature
      const geometry = feature.getGeometry();
      if (!geometry) return;
      const geometryType = geometry.getType();
      let baseName = "";
      let namePrefix = "";

      if (geometryType === "Point") {
        baseName = "Станция";
        namePrefix = "Станция ";
      } else if (geometryType === "LineString") {
        baseName = "Маршрут";
        namePrefix = "Маршрут ";
      } else {
        baseName = "Объект";
        namePrefix = "Объект ";
      }

      const existingNamedFeatures = this.vectorSource
        .getFeatures()
        .filter(
          (f) =>
            f !== feature &&
            f.getGeometry()?.getType() === geometryType &&
            f.get("name") &&
            (f.get("name") as string).startsWith(namePrefix)
        );

      let maxNumber = 0;
      existingNamedFeatures.forEach((f) => {
        const name = f.get("name") as string;
        if (name) {
          const numStr = name.substring(namePrefix.length);
          const num = parseInt(numStr, 10);
          if (!isNaN(num) && num > maxNumber) {
            maxNumber = num;
          }
        }
      });

      const newNumber = maxNumber + 1;
      feature.set("name", `${baseName} ${newNumber}`);
      feature.setStyle(
        type === "Point" ? this.busIconStyle : this.defaultStyle
      );
      if (type === "LineString") this.finishDrawing();
    });
    this.map.addInteraction(this.currentInteraction);
  }

  public startDrawingMarker(): void {
    this.startDrawing("Point");
  }

  public startDrawingLine(): void {
    this.startDrawing("LineString");
  }

  public finishDrawing(): void {
    if (!this.map || !this.currentInteraction) return;

    const drawInteraction = this.currentInteraction as Draw;
    if (
      (drawInteraction as unknown as { sketchFeature_?: Feature<Geometry> })
        .sketchFeature_
    ) {
      try {
        // This can throw if not enough points for geometry (e.g. LineString)
        this.currentInteraction.finishDrawing();
      } catch (e) {
        // console.warn("Could not finish drawing programmatically:", e);
        // If finishDrawing fails (e.g. LineString with one point),
        // we still want to remove interaction and reset mode.
      }
    }
    this.map.removeInteraction(this.currentInteraction);
    this.currentInteraction = null;
    this.currentDrawingType = null;
    this.activateEditMode();
  }

  private handleContextMenu(event: MouseEvent): void {
    event.preventDefault();
    if (
      !this.map ||
      !(
        this.mode &&
        this.mode.startsWith("drawing-") &&
        this.currentInteraction instanceof Draw
      )
    ) {
      return;
    }

    const drawInteraction = this.currentInteraction as Draw;
    if (
      this.currentDrawingType === "LineString" &&
      (drawInteraction as unknown as { sketchFeature_?: Feature<Geometry> })
        .sketchFeature_
    ) {
      try {
        this.currentInteraction.finishDrawing();
      } catch (e) {
        this.finishDrawing(); // Fallback to ensure cleanup
      }
    } else {
      this.finishDrawing();
    }
  }

  private handlePointerMove(event: MapBrowserEvent<PointerEvent>): void {
    if (!this.map || event.dragging) {
      if (this.hoveredFeatureId) {
        this.hoveredFeatureId = null;
        this.vectorLayer.changed();
      }
      if (this.tooltipOverlay) this.tooltipOverlay.setPosition(undefined);
      return;
    }

    const pixel = this.map.getEventPixel(event.originalEvent as PointerEvent);
    const featureAtPixel: Feature<Geometry> | undefined =
      this.map.forEachFeatureAtPixel(
        pixel,
        (f: FeatureLike) => f as Feature<Geometry>, // Cast is okay due to layerFilter
        {
          layerFilter: (l) => l === this.vectorLayer,
          hitTolerance: 5,
        }
      );

    const newHoveredFeatureId = featureAtPixel
      ? featureAtPixel.getId()
      : (null as string | number | null);

    if (this.tooltipOverlay && this.tooltipElement) {
      if (
        this.mode === "statistics" &&
        featureAtPixel &&
        featureAtPixel.getGeometry()?.getType() === "Point"
      ) {
        this.tooltipElement.innerHTML =
          (featureAtPixel.get("name") as string) || "Станция";
        this.tooltipOverlay.setPosition(event.coordinate);
      } else {
        this.tooltipOverlay.setPosition(undefined);
      }
    }

    if (this.hoveredFeatureId !== newHoveredFeatureId) {
      this.hoveredFeatureId = newHoveredFeatureId as string | number | null;
      this.vectorLayer.changed();
    }
  }

  private handleClick(event: MapBrowserEvent<PointerEvent>): void {
    if (!this.map || (this.mode && this.mode.startsWith("drawing-"))) return;

    const featureAtPixel: Feature<Geometry> | undefined =
      this.map.forEachFeatureAtPixel(
        event.pixel,
        (f: FeatureLike) => f as Feature<Geometry>,
        {
          layerFilter: (l) => l === this.vectorLayer,
          hitTolerance: 2, // Lower tolerance for click
        }
      );

    if (this.mode === "statistics") {
      this.selectInteraction.getFeatures().clear(); // Ensure no edit selection in stats mode
      if (featureAtPixel) {
        const featureId = featureAtPixel.getId();
        if (this.infoSelectedFeatureId !== featureId) {
          this.infoSelectedFeatureId = featureId as string | number | null;
          this.getFeatureCoordinates(featureAtPixel);
          this.onFeatureSelect(featureAtPixel);
        } else {
          // Clicked same feature again, deselect
          this.infoSelectedFeatureId = null;
          this.hideCoordinatesPanel();
          this.onFeatureSelect(null);
        }
      } else {
        // Clicked on map, not on a feature
        this.infoSelectedFeatureId = null;
        this.hideCoordinatesPanel();
        this.onFeatureSelect(null);
      }
      this.vectorLayer.changed(); // Re-render for style changes
    } else if (this.mode === "edit") {
      // Selection in edit mode is handled by the Select interaction's 'select' event
      // This click handler can ensure that if a click happens outside any feature
      // (and doesn't trigger a new selection by the Select interaction),
      // any info selection from a previous mode is cleared.
      if (this.infoSelectedFeatureId) {
        this.infoSelectedFeatureId = null;
        this.vectorLayer.changed();
      }
      // If click was on a feature, Select interaction handles it.
      // If click was on map, Select interaction's 'select' event with e.deselected might clear panel.
      // If no feature is selected after this click, the coordinates panel should be hidden by Select's logic.
      if (
        !featureAtPixel &&
        this.selectInteraction.getFeatures().getLength() === 0
      ) {
        this.hideCoordinatesPanel();
        this.onFeatureSelect(null);
      }
    }
  }

  public selectFeature(featureId: string | number | undefined): void {
    if (!this.map || featureId === undefined) {
      this.onFeatureSelect(null);
      if (this.mode === "edit") this.selectInteraction.getFeatures().clear();
      if (this.mode === "statistics" && this.infoSelectedFeatureId) {
        this.infoSelectedFeatureId = null;
        this.vectorLayer.changed();
      }
      this.hideCoordinatesPanel();
      return;
    }

    const feature = this.vectorSource.getFeatureById(featureId);

    if (!feature) {
      this.onFeatureSelect(null);
      if (this.mode === "edit") this.selectInteraction.getFeatures().clear();
      if (
        this.mode === "statistics" &&
        this.infoSelectedFeatureId === featureId
      ) {
        this.infoSelectedFeatureId = null;
      }
      this.hideCoordinatesPanel();
      this.vectorLayer.changed();
      return;
    }

    if (this.mode === "edit") {
      this.infoSelectedFeatureId = null; // Clear any info selection
      this.selectInteraction.getFeatures().clear();
      this.selectInteraction.getFeatures().push(feature);
      // The 'select' event on selectInteraction should handle getFeatureCoordinates and onFeatureSelect
    } else if (this.mode === "statistics") {
      this.selectInteraction.getFeatures().clear(); // Clear any edit selection
      this.infoSelectedFeatureId = featureId;
      this.getFeatureCoordinates(feature);
      this.onFeatureSelect(feature);
    }
    this.vectorLayer.changed();

    const view = this.map.getView();
    const geometry = feature.getGeometry();
    if (geometry) {
      if (geometry.getType() === "Point") {
        const pointGeom = geometry as Point;
        view.animate({
          center: pointGeom.getCoordinates(),
          duration: 500,
          zoom: Math.max(view.getZoom() || mapConfig.zoom, 14),
        });
      } else {
        view.fit(geometry.getExtent(), {
          duration: 500,
          padding: [50, 50, 50, 50],
          maxZoom: 15,
        });
      }
    }
  }

  public deleteFeature(featureId: string | number | undefined): void {
    if (featureId === undefined) return;
    const feature = this.vectorSource.getFeatureById(featureId);
    if (feature) {
      const currentState = this.getCurrentStateAsGeoJSON();
      if (currentState) {
        this.addStateToHistory("delete", currentState);
      }

      if (this.infoSelectedFeatureId === featureId) {
        this.infoSelectedFeatureId = null;
        this.onFeatureSelect(null); // Notify React that selection changed
      }

      const selectedFeaturesCollection = this.selectInteraction?.getFeatures();
      if (selectedFeaturesCollection?.getArray().includes(feature)) {
        selectedFeaturesCollection.clear();
        // This will trigger selectInteraction's 'select' event with deselected,
        // which should handle UI updates like hiding coordinates panel.
        this.onFeatureSelect(null); // Explicitly notify too
      }

      this.vectorSource.removeFeature(feature); // This will trigger 'removefeature' event

      // If after deletion, no feature is selected in any mode, hide panel
      if (
        !this.infoSelectedFeatureId &&
        selectedFeaturesCollection?.getLength() === 0
      ) {
        this.hideCoordinatesPanel();
      }
      this.vectorLayer.changed(); // Ensure map re-renders
    }
  }

  private getFeatureCoordinates(feature: Feature<Geometry>): void {
    if (!feature || !feature.getGeometry()) {
      this.hideCoordinatesPanel();
      return;
    }
    const geometry = feature.getGeometry();
    if (!geometry) {
      this.hideCoordinatesPanel();
      return;
    }

    const toGeoCoords = (c: number[]) => transform(c, "EPSG:3857", "EPSG:4326");
    const fmt = new Intl.NumberFormat("ru-RU", {
      minimumFractionDigits: 5,
      maximumFractionDigits: 5,
    });
    let txt = "";
    const fType = geometry.getType();
    const fName =
      (feature.get("name") as string) ||
      (fType === "Point"
        ? "Станция"
        : fType === "LineString"
        ? "Маршрут"
        : "Объект");

    let nameClr = "text-blue-600";
    if (
      this.mode === "statistics" &&
      this.infoSelectedFeatureId === feature.getId()
    ) {
      nameClr = "text-red-600";
    } else if (
      this.mode === "edit" &&
      this.selectInteraction?.getFeatures().getArray().includes(feature)
    ) {
      nameClr = "text-orange-600";
    }
    txt += `<strong class="${nameClr}">${fName}:</strong><br>`;

    if (geometry instanceof Point) {
      const crds = geometry.getCoordinates();
      const [lon, lat] = toGeoCoords(crds);
      txt += `Шир: ${fmt.format(lat)}, Дол: ${fmt.format(lon)}`;
    } else if (geometry instanceof LineString) {
      const crdsArr = geometry.getCoordinates();
      txt += crdsArr
        .map((c, i) => {
          const [lon, lat] = toGeoCoords(c);
          return `<span class="text-xs">Т${i + 1}:</span> Ш: ${fmt.format(
            lat
          )}, Д: ${fmt.format(lon)}`;
        })
        .join("<br>");
      const lenM = geometry.getLength();
      txt += `<br><span class="text-xs">Длина:</span> ${lenM.toFixed(2)} м`;
    } else {
      txt += "Координаты недоступны.";
    }

    this.setCoordinatesPanelContent(txt);
    this.setShowCoordinatesPanel(true);
  }

  private hideCoordinatesPanel(): void {
    this.setCoordinatesPanelContent("");
    this.setShowCoordinatesPanel(false);
  }

  public getAllFeaturesAsGeoJSON(): string | null {
    if (!this.vectorSource || !this.map) return null;
    const feats = this.vectorSource.getFeatures();
    if (feats.length === 0) return null;

    const geoJSONFmt = new GeoJSON();
    // Clone features to avoid transforming original geometries
    const featsExp = feats.map((f) => {
      const cF = f.clone();
      // Make sure to copy properties and ID
      cF.setProperties(f.getProperties(), true); // true to suppress change event
      cF.setId(f.getId());
      const geom = cF.getGeometry();
      if (geom) {
        geom.transform(this.map!.getView().getProjection(), "EPSG:4326");
      }
      return cF;
    });
    return geoJSONFmt.writeFeatures(featsExp);
  }

  public destroy(): void {
    if (this.map) {
      document.removeEventListener("keydown", this.boundHandleKeyDown);
      const targetEl = this.map.getTargetElement();
      if (targetEl instanceof HTMLElement) {
        targetEl.removeEventListener(
          "contextmenu",
          this.boundHandleContextMenu
        );
        targetEl.removeEventListener(
          "pointerleave",
          this.boundHandlePointerLeave
        );
      }
      this.map.un("pointermove", this.boundHandlePointerMove as any);
      if (this.tooltipOverlay) this.map.removeOverlay(this.tooltipOverlay);
      this.vectorSource.clear();
      this.map.setTarget(undefined); // Use undefined instead of null
      this.map = null;
    }
  }

  private handleFeatureEvent(
    event: VectorSourceEvent<Feature<Geometry>>
  ): void {
    if (!event.feature) return;
    const feature = event.feature;
    if (!feature.getId()) {
      feature.setId(Date.now() + Math.random().toString(36).substr(2, 9));
    }
    this.updateFeaturesInReact();
  }

  private handleFeatureChange(
    event: VectorSourceEvent<Feature<Geometry>>
  ): void {
    if (!event.feature) return;
    const feature = event.feature;
    this.updateFeaturesInReact();

    const selectedInEdit = this.selectInteraction
      ?.getFeatures()
      .getArray()
      .includes(feature);
    const selectedInInfo = this.infoSelectedFeatureId === feature.getId();

    if (
      (this.mode === "edit" && selectedInEdit) ||
      (this.mode === "statistics" && selectedInInfo)
    ) {
      this.getFeatureCoordinates(feature);
    }
  }
}

// --- MAP CONTROLS COMPONENT ---
interface MapControlsProps {
  mapService: MapService | null;
  activeMode: string;
}

const MapControls: React.FC<MapControlsProps> = ({
  mapService,
  activeMode,
}) => {
  if (!mapService) return null;

  const controls = [
    {
      mode: "edit",
      title: "Редактировать",
      longTitle: "Редактирование",
      icon: <EditIcon />,
      action: () => mapService.activateEditMode(),
    },
    {
      mode: "statistics",
      title: "Информация",
      longTitle: "Информация",
      icon: <StatsIcon />,
      action: () => mapService.activateStatisticsMode(),
    },
    {
      mode: "drawing-point",
      title: "Станция",
      longTitle: "Добавить станцию",
      icon: <Bus size={16} className="mr-1 sm:mr-2" />,
      action: () => mapService.startDrawingMarker(),
    },
    {
      mode: "drawing-linestring",
      title: "Маршрут",
      longTitle: "Добавить маршрут",
      icon: <LineIconSvg />,
      action: () => mapService.startDrawingLine(),
    },
  ];
  return (
    <div className="absolute  top-4 left-1/2 -translate-x-1/2 z-20 flex flex-nowrap justify-center p-2 bg-white/90 backdrop-blur-sm rounded-lg shadow-xl space-x-1 sm:space-x-2">
      {controls.map((c) => (
        <button
          key={c.mode}
          className={`flex items-center px-3 py-2 rounded-md transition-all duration-200 text-xs sm:text-sm font-medium focus:outline-none focus:ring-2 focus:ring-offset-1 focus:ring-blue-500 focus:ring-opacity-75 ${
            activeMode === c.mode
              ? "bg-blue-600 text-white shadow-md hover:bg-blue-700"
              : "bg-gray-100 text-gray-700 hover:bg-blue-100 hover:text-blue-700"
          }`}
          onClick={c.action}
          title={c.longTitle}
        >
          {c.icon}
          <span className="hidden sm:inline ml-0 sm:ml-1">{c.title}</span>
        </button>
      ))}
    </div>
  );
};

// --- MAP SIGHTBAR COMPONENT ---
interface MapSightbarProps {
  mapService: MapService | null;
  mapFeatures: Feature<Geometry>[];
  selectedFeature: Feature<Geometry> | null;
  currentMapMode: string;
}

const MapSightbar: React.FC<MapSightbarProps> = ({
  mapService,
  mapFeatures,
  selectedFeature,
  currentMapMode,
}) => {
  const [activeSection, setActiveSection] = useState<string | null>("layers");
  const toggleSection = (id: string) =>
    setActiveSection(activeSection === id ? null : id);

  const handleFeatureClick = useCallback(
    (id: string | number | undefined) => {
      if (mapService) mapService.selectFeature(id);
    },
    [mapService]
  );

  const handleDeleteFeature = useCallback(
    (id: string | number | undefined) => {
      if (
        mapService &&
        window.confirm("Вы действительно хотите удалить этот объект?")
      )
        mapService.deleteFeature(id);
    },
    [mapService]
  );

  const stations = mapFeatures.filter(
    (f) => f.getGeometry()?.getType() === "Point"
  );
  const lines = mapFeatures.filter(
    (f) => f.getGeometry()?.getType() === "LineString"
  );

  interface SidebarSection {
    id: string;
    title: string;
    icon: ReactNode;
    content: ReactNode;
  }

  const sections: SidebarSection[] = [
    {
      id: "layers",
      title: `Остановки (${stations.length})`,
      icon: <Bus size={20} />,
      content: (
        <div className="space-y-1 max-h-60 overflow-y-auto pr-1">
          {stations.length > 0 ? (
            stations.map((s) => {
              const sId = s.getId();
              const sName = (s.get("name") as string) || "Без названия";
              const isSelectedInCurrentMode = selectedFeature?.getId() === sId;
              const sGeom = s.getGeometry();
              let sCoordsText: string | null = null;

              if (sGeom instanceof Point) {
                const sC = transform(
                  sGeom.getCoordinates(),
                  "EPSG:3857",
                  "EPSG:4326"
                );
                const cF = new Intl.NumberFormat("ru-RU", {
                  minimumFractionDigits: 3,
                  maximumFractionDigits: 3,
                });
                sCoordsText = `${cF.format(sC[1])}, ${cF.format(sC[0])}`;
              }

              return (
                <div
                  key={String(sId)} // Ensure key is string
                  className={`flex items-start justify-between p-2 rounded-md cursor-pointer group transition-colors duration-150 ${
                    isSelectedInCurrentMode && currentMapMode === "statistics"
                      ? "bg-red-100 border border-red-300 hover:bg-red-200"
                      : isSelectedInCurrentMode && currentMapMode === "edit"
                      ? "bg-orange-100 border border-orange-300 hover:bg-orange-200"
                      : "hover:bg-blue-50"
                  }`}
                  onClick={() => handleFeatureClick(sId)}
                >
                  <div className="flex flex-col text-gray-800 text-sm flex-grow mr-2">
                    <div className="flex items-center mb-0.5">
                      <MapPin
                        size={16}
                        className={`mr-1.5 flex-shrink-0 ${
                          isSelectedInCurrentMode &&
                          currentMapMode === "statistics"
                            ? "text-red-500"
                            : isSelectedInCurrentMode &&
                              currentMapMode === "edit"
                            ? "text-orange-500"
                            : "text-blue-500 group-hover:text-blue-600"
                        }`}
                      />
                      <span
                        className={`font-medium truncate ${
                          isSelectedInCurrentMode &&
                          currentMapMode === "statistics"
                            ? "text-red-600"
                            : isSelectedInCurrentMode &&
                              currentMapMode === "edit"
                            ? "text-orange-600"
                            : "group-hover:text-blue-600"
                        }`}
                        title={sName}
                      >
                        {sName}
                      </span>
                    </div>
                    {sCoordsText && (
                      <p className="text-xs text-gray-500 ml-6 group-hover:text-gray-600">
                        {sCoordsText}
                      </p>
                    )}
                  </div>
                  <button
                    onClick={(e) => {
                      e.stopPropagation();
                      handleDeleteFeature(sId);
                    }}
                    className="p-1 rounded-full text-gray-400 hover:text-red-600 hover:bg-red-100 transition-colors flex-shrink-0 opacity-60 group-hover:opacity-100"
                    title="Удалить остановку"
                  >
                    <Trash2 size={16} />
                  </button>
                </div>
              );
            })
          ) : (
            <p className="text-gray-500 text-sm">Нет добавленных остановок.</p>
          )}
        </div>
      ),
    },
    {
      id: "lines",
      title: `Маршруты (${lines.length})`,
      icon: <RouteIcon size={20} />,
      content: (
        <div className="space-y-1 max-h-60 overflow-y-auto pr-1">
          {lines.length > 0 ? (
            lines.map((l) => {
              const lId = l.getId();
              const lName = (l.get("name") as string) || "Без названия";
              const isSelectedInCurrentMode = selectedFeature?.getId() === lId;

              return (
                <div
                  key={String(lId)} // Ensure key is string
                  className={`flex items-center justify-between p-2 rounded-md cursor-pointer group transition-colors duration-150 ${
                    isSelectedInCurrentMode && currentMapMode === "statistics"
                      ? "bg-red-100 border border-red-300 hover:bg-red-200"
                      : isSelectedInCurrentMode && currentMapMode === "edit"
                      ? "bg-orange-100 border border-orange-300 hover:bg-orange-200"
                      : "hover:bg-blue-50"
                  }`}
                  onClick={() => handleFeatureClick(lId)}
                >
                  <div className="flex items-center text-gray-800 text-sm flex-grow mr-2">
                    <ArrowRightLeft
                      size={16}
                      className={`mr-1.5 flex-shrink-0 ${
                        isSelectedInCurrentMode &&
                        currentMapMode === "statistics"
                          ? "text-red-500"
                          : isSelectedInCurrentMode && currentMapMode === "edit"
                          ? "text-orange-500"
                          : "text-purple-500 group-hover:text-purple-600"
                      }`}
                    />
                    <span
                      className={`font-medium truncate ${
                        isSelectedInCurrentMode &&
                        currentMapMode === "statistics"
                          ? "text-red-600"
                          : isSelectedInCurrentMode && currentMapMode === "edit"
                          ? "text-orange-600"
                          : "group-hover:text-purple-600"
                      }`}
                      title={lName}
                    >
                      {lName}
                    </span>
                  </div>
                  <button
                    onClick={(e) => {
                      e.stopPropagation();
                      handleDeleteFeature(lId);
                    }}
                    className="p-1 rounded-full text-gray-400 hover:text-red-600 hover:bg-red-100 transition-colors flex-shrink-0 opacity-60 group-hover:opacity-100"
                    title="Удалить маршрут"
                  >
                    <Trash2 size={16} />
                  </button>
                </div>
              );
            })
          ) : (
            <p className="text-gray-500 text-sm">Нет добавленных маршрутов.</p>
          )}
        </div>
      ),
    },
  ];

  if (!sections.length && activeSection) {
    setActiveSection(null);
  } else if (
    sections.length > 0 &&
    !sections.find((s) => s.id === activeSection)
  ) {
    setActiveSection(sections[0]?.id || null);
  }

  return (
    <div className="w-72 md:w-80 bg-gray-50 shadow-lg flex flex-col border-l border-gray-200 h-screen">
      <div className="p-4 bg-gray-700 text-white">
        <h2 className="text-lg font-semibold">Панель управления</h2>
      </div>
      <div className="flex-1 overflow-y-auto">
        {sections.map((s) => (
          <div key={s.id} className="border-b border-gray-200 last:border-b-0">
            <button
              onClick={() => toggleSection(s.id)}
              className={`w-full p-3 flex items-center justify-between text-left hover:bg-gray-100 transition-colors duration-150 focus:outline-none focus:bg-gray-100 ${
                activeSection === s.id
                  ? "bg-gray-100 text-blue-600"
                  : "text-gray-700"
              }`}
            >
              <div className="flex items-center space-x-3">
                <span
                  className={
                    activeSection === s.id ? "text-blue-600" : "text-gray-600"
                  }
                >
                  {s.icon}
                </span>
                <span className="font-medium text-sm">{s.title}</span>
              </div>
              <span
                className={`transform transition-transform duration-200 text-gray-500 ${
                  activeSection === s.id ? "rotate-180" : ""
                }`}
              >
                ▼
              </span>
            </button>
            <div
              className={`overflow-hidden transition-all duration-300 ease-in-out ${
                activeSection === s.id
                  ? "max-h-[600px] opacity-100" // Adjusted max-h to be generous
                  : "max-h-0 opacity-0"
              }`}
            >
              <div className="p-3 text-sm text-gray-600 bg-white border-t border-gray-100">
                {s.content}
              </div>
            </div>
          </div>
        ))}
      </div>
    </div>
  );
};

// --- MAP PAGE COMPONENT ---
export const MapPage: React.FC = () => {
  const mapRef = useRef<HTMLDivElement | null>(null);
  const tooltipRef = useRef<HTMLDivElement | null>(null);
  const [mapServiceInstance, setMapServiceInstance] =
    useState<MapService | null>(null);
  const [coordinatesPanelContent, setCoordinatesPanelContent] =
    useState<string>("");
  const [showCoordinatesPanel, setShowCoordinatesPanel] =
    useState<boolean>(false);
  const [isLoading, setIsLoading] = useState<boolean>(true);
  const [error, setError] = useState<string | null>(null);
  const [currentMapMode, setCurrentMapMode] = useState<string>("edit"); // Default mode
  const [mapFeatures, setMapFeatures] = useState<Feature<Geometry>[]>([]);
  const [selectedFeatureForSidebar, setSelectedFeatureForSidebar] =
    useState<Feature<Geometry> | null>(null);

  const handleFeaturesChange = useCallback(
    (feats: Feature<Geometry>[]) => setMapFeatures([...feats]), // Create new array instance
    []
  );

  const handleFeatureSelectForSidebar = useCallback(
    (feat: Feature<Geometry> | null) => {
      setSelectedFeatureForSidebar(feat);
      if (!feat && showCoordinatesPanel) {
        // Check showCoordinatesPanel state before calling setter
        setShowCoordinatesPanel(false);
      }
    },
    [showCoordinatesPanel] // Dependency on showCoordinatesPanel as it's read
  );

  useEffect(() => {
    let service: MapService | null = null; // Initialize to null
    if (mapRef.current && tooltipRef.current && !mapServiceInstance) {
      setIsLoading(true);
      setError(null);
      try {
        service = new MapService(
          { ...mapConfig, target: mapRef.current },
          setCoordinatesPanelContent,
          setShowCoordinatesPanel,
          setIsLoading,
          setError,
          setCurrentMapMode,
          handleFeaturesChange,
          handleFeatureSelectForSidebar,
          tooltipRef.current
        );
        setMapServiceInstance(service);
      } catch (e: any) {
        console.error("MapPage useEffect error:", e);
        setError(
          `Ошибка инициализации карты: ${
            e.message || "Неизвестная ошибка"
          }. Пожалуйста, проверьте консоль.`
        );
        setIsLoading(false);
      }
    }
    return () => {
      if (service) {
        service.destroy();
        setMapServiceInstance(null); // Ensure instance is cleared on unmount
      }
    };
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []); // Added stable useCallback refs

  return (
    <div className="flex flex-col md:flex-row font-sans bg-gray-100 h-[80vh] overflow-hidden">
      <div className="relative flex-grow flex">
        <div
          ref={mapRef}
          id="map"
          className="flex-1 h-full relative bg-gray-200"
        >
          <div
            ref={tooltipRef}
            className="tooltip ol-tooltip bg-white text-black p-2 rounded shadow-lg text-xs whitespace-nowrap"
            style={{
              position: "absolute",
              pointerEvents: "none",
              // Ensure it's hidden initially if empty or not positioned by OL
              visibility: "hidden",
            }}
          ></div>

          {isLoading && (
            <div className="absolute inset-0 flex flex-col items-center justify-center bg-gray-500 bg-opacity-50 z-[1001]">
              <div className="w-12 h-12 border-4 border-blue-500 border-t-transparent rounded-full animate-spin mb-3"></div>
              <div className="text-md font-semibold text-white drop-shadow-md">
                Загрузка карты...
              </div>
            </div>
          )}
          {error && !isLoading && (
            <div className="absolute top-1/2 left-1/2 -translate-x-1/2 -translate-y-1/2 bg-red-100 border border-red-400 text-red-700 p-6 rounded-lg shadow-lg z-[1002] text-center max-w-md">
              <h3 className="font-semibold text-lg mb-2">Произошла ошибка</h3>
              <p className="text-sm">{error}</p>
              <button
                onClick={() => window.location.reload()}
                className="mt-4 px-4 py-2 bg-red-500 text-white rounded hover:bg-red-600 transition-colors text-sm"
              >
                Перезагрузить страницу
              </button>
            </div>
          )}
        </div>
        {mapServiceInstance && !isLoading && !error && (
          <MapControls
            mapService={mapServiceInstance}
            activeMode={currentMapMode}
          />
        )}
        <div
          className={`absolute bottom-4 left-1/2 -translate-x-1/2 bg-white/90 backdrop-blur-sm p-3 rounded-lg shadow-xl z-20 max-w-[calc(100%-1rem)] sm:max-w-md text-left text-xs sm:text-sm text-gray-800 leading-relaxed transition-all duration-300 max-h-[150px] overflow-y-auto ${
            showCoordinatesPanel &&
            coordinatesPanelContent &&
            !isLoading &&
            !error
              ? "opacity-100 visible translate-y-0"
              : "opacity-0 invisible translate-y-4"
          }`}
        >
          <div dangerouslySetInnerHTML={{ __html: coordinatesPanelContent }} />
        </div>
      </div>
      {mapServiceInstance && !isLoading && !error && (
        <MapSightbar
          mapService={mapServiceInstance}
          mapFeatures={mapFeatures}
          selectedFeature={selectedFeatureForSidebar}
          currentMapMode={currentMapMode}
        />
      )}
    </div>
  );
};
